==> tecfy_core_package-0.1.36.tar.gz: lib/models/entity/property_dropdown.model.dart <==
part of tecfy_core_client_package;

class PropertyDropdown extends Property {
  String entityName;

  /// <summary>
  /// used to get the list using custom EndPoint like "Item/GetItemInfo?brand={brandId}&model={modelId}&category={itemCategoryId}&year={year}"
  /// </summary>
  String dropdownUrl;

  bool isFilter;

  /// <summary>
  /// where value of FilterField is null of embty then the value of FilterValueField must refer to Array proerty of same dropdwn Entity
  /// </summary>
  String filterField;

  /// <summary>
  /// where value of FilterField is null of embty then the value of FilterValueField must refer to Array proerty of same dropdwn Entity
  /// </summary>
  String filterValueField;

  /// <summary>
  /// Fileds that in main entity and dropdown entity with the same name, that you need to update the value of the main entity with the value in dropdown entity
  /// For example: when select item the UOM_ID of the invoice will be overwrten
  /// Can add multi properties seperated by comma
  /// </summary>
  List<String> overwritePropertiesOnSelect;

  PropertyDropdown(
      {dbName,
      nameAr,
      nameEn,
      isRequired,
      this.entityName,
      this.overwritePropertiesOnSelect,
      this.isFilter = false,
      this.filterField,
      this.filterValueField})
      : super(dbName: dbName, nameAr: nameAr, nameEn: nameEn,isRequired:isRequired) {
    super.propertyType = PropertyTypes.Dropdown;
  }

  PropertyDropdown.fromMap(Map row) : super.fromMap(row) {
    entityName = row['entityName'];
    dropdownUrl = row['dropdownUrl'];
    isFilter = row['isFilter'];
    filterField = lowerFirst(row['filterField']);
    filterValueField = lowerFirst(row['filterValueField']);
    if (row['overwritePropertiesOnSelect'] != null) {
      overwritePropertiesOnSelect = row['overwritePropertiesOnSelect']
          .toString()
          .split(',')
          .map((d) => lowerFirst(d))
          .toList();
    } else {
      overwritePropertiesOnSelect = List.empty();
    }
  }
  @override
  OperationResult isValid(data) {
    if (!super.isValid(data).success) return super.isValid(data);

    return OperationResult(success: true);
  }

  Widget getDisplayWidget(Map data,
      {double size = 12, Color color = Colors.black}) {
    return Text("[Get dropdown data]");
  }

  Future<bool> getDefaultValue(Map<String, dynamic> data) async {
    //if(data[dbName] != null) return false;

    var dbNameEntity = dbName.substring(0, dbName.indexOf('Id'));
    var dbNameParent = dbName + '______Parent';
    try {
      //print('==================== set default value of dropdown =============== $dbName   $dbNameEntity');
      // Get the default value and set it
      if (defaultString.isNotEmpty) {
        if (int.tryParse(defaultString) != null) {
          // TODO: fix after you make Id string
          if (data[dbName] == null || data[dbName] == 0) {
            data[dbName] = int.parse(defaultString);
            return true;
          }
        } else {
          if (defaultString.contains('.')) {
            var parentPropName = defaultString.split('.')[0];
            var childPropName = defaultString.split('.')[1];

            var parentData = data[lowerFirst(parentPropName)];
            if (data[dbNameParent] != null &&
                data[dbNameParent]['id'] == parentData['id']) return false;
            data[dbNameParent] = parentData;
            //print('------ Parent Data $parentPropName $parentData');
            var childData = parentData[lowerFirst(childPropName)];
            //print('------ Child Data $childPropName $childData' );
            if (childData is int) {
              if (data[dbName] != childData) {
                data[dbName] = childData;
              }
            } else {
              // TODO: hadel this case
            }
          } else {
            // TODO: hadel this case
          }
        }

        if (data[dbName] != null &&
            data[dbName] != 0 &&
            (data[dbNameEntity] == null ||
                data[dbNameEntity]["id"] != data[dbName])) {
          print(
              '-------- get data for value ${data[dbName]} ${data[dbNameEntity] == null ? 'null' : data[dbNameEntity]['id']}');
          var result = await EntityService().getOne(entityName, data[dbName]);
          data[dbNameEntity] = result;
          if (data[dbNameEntity] != null) return true;
        }
      }
    } catch (err) {
      print(
          'XXXXXXXXXXXXxxxxxxxxxxxx ERROR  getDefaultValue for Dropdown $dbName: $err');
    }
    return false;
  }
}

==> tecfy_core_package-0.1.55.tar.gz: lib/models/entity/property_dropdown.model.dart <==
part of tecfy_core_client_package;

class PropertyDropdown extends Property {
  String entityName;
  String dbNameEntity;
  Property entityDisplayProp;

  /// <summary>
  /// used to get the list using custom EndPoint like "Item/GetItemInfo?brand={brandId}&model={modelId}&category={itemCategoryId}&year={year}"
  /// </summary>
  String dropdownUrl;

  bool isFilter;

  /// <summary>
  /// where value of FilterField is null of embty then the value of FilterValueField must refer to Array proerty of same dropdwn Entity
  /// </summary>
  String filterField;
  String filterQuery;

  /// <summary>
  /// where value of FilterField is null of embty then the value of FilterValueField must refer to Array proerty of same dropdwn Entity
  /// </summary>
  String filterValueField;

  /// <summary>
  /// Fileds that in main entity and dropdown entity with the same name, that you need to update the value of the main entity with the value in dropdown entity
  /// For example: when select item the UOM_ID of the invoice will be overwrten
  /// Can add multi properties seperated by comma
  /// </summary>
  List<String> overwritePropertiesOnSelect;

  PropertyDropdown(
      {dbName,
      nameAr,
      nameEn,
      isRequired,
      this.entityName,
      this.overwritePropertiesOnSelect,
      this.isFilter = false,
      this.filterField,
      this.filterQuery,
      this.filterValueField})
      : super(
            dbName: dbName,
            nameAr: nameAr,
            nameEn: nameEn,
            isRequired: isRequired) {
    super.propertyType = PropertyTypes.Dropdown;
    if (this.dbName != null)
      dbNameEntity = this.dbName.substring(0, this.dbName.length - 2);
  }

  PropertyDropdown.fromMap(Map row) : super.fromMap(row) {
    entityName = row['entityName'];
    dropdownUrl = row['dropdownUrl'];
    isFilter = row['isFilter'];
    filterField = lowerFirst(row['filterField']);
    filterQuery = row['filterQuery'] ?? '';
    filterValueField = lowerFirst(row['filterValueField']);
    if (row['overwritePropertiesOnSelect'] != null) {
      overwritePropertiesOnSelect = row['overwritePropertiesOnSelect']
          .toString()
          .split(',')
          .map((d) => lowerFirst(d))
          .toList();
    } else {
      overwritePropertiesOnSelect = List.empty();
    }
    if (this.dbName != null)
      dbNameEntity = this.dbName.substring(0, this.dbName.length - 2);
  }
  @override
  OperationResult isValid(data) {
    if (!super.isValid(data).success) return super.isValid(data);

    return OperationResult(success: true);
  }

  Widget getDisplayWidget(Map data,
      {double size = 12, Color color = Colors.black}) {
    if (this.dbNameEntity == null)
      dbNameEntity = this.dbName.substring(0, this.dbName.length - 2);
    if (entityDisplayProp == null) {
      entityDisplayProp = EntityService.getEntity(entityName).displayField;
    }
    return Text(
        data[this.dbNameEntity] == null
            ? '-'
            : '${data[this.dbNameEntity][entityDisplayProp.dbName]}',
        style: TextStyle(fontSize: size, color: color));
  }

  Future<bool> getDefaultValue(Map<String, dynamic> data) async {
    //if(data[dbName] != null) return false;

    var dbNameEntity = dbName.substring(0, dbName.indexOf('Id'));
    var dbNameParent = dbName + '______Parent';
    try {
      //print('==================== set default value of dropdown =============== $dbName   $dbNameEntity');
      // Get the default value and set it
      if (defaultString.isNotEmpty) {
        if (int.tryParse(defaultString) != null) {
          // TODO: fix after you make Id string
          if (data[dbName] == null || data[dbName] == 0) {
            data[dbName] = int.parse(defaultString);
            return true;
          }
        } else {
          if (defaultString.contains('.')) {
            var parentPropName = defaultString.split('.')[0];
            var childPropName = defaultString.split('.')[1];

            var parentData = data[lowerFirst(parentPropName)];
            if (data[dbNameParent] != null &&
                data[dbNameParent]['id'] == parentData['id']) return false;
            data[dbNameParent] = parentData;
            //print('------ Parent Data $parentPropName $parentData');
            var childData = parentData[lowerFirst(childPropName)];
            //print('------ Child Data $childPropName $childData' );
            if (childData is int) {
              if (data[dbName] != childData) {
                data[dbName] = childData;
              }
            } else {
              // TODO: hadel this case
            }
          } else {
            // TODO: hadel this case
          }
        }

        if (data[dbName] != null &&
            data[dbName] != 0 &&
            (data[dbNameEntity] == null ||
                data[dbNameEntity]["id"] != data[dbName])) {
          print(
              '-------- get data for value ${data[dbName]} ${data[dbNameEntity] == null ? 'null' : data[dbNameEntity]['id']}');
          var result = await EntityService.getOne(entityName, data[dbName],
              withRelatedData: true);
          data[dbNameEntity] = result;
          if (data[dbNameEntity] != null) return true;
        }
      }
    } catch (err) {
      print(
          'XXXXXXXXXXXXxxxxxxxxxxxx ERROR  getDefaultValue for Dropdown $dbName: $err');
    }
    return false;
  }
}
